@echo off
set MQF_LOGECHOFILE=N

rem ---------------------------------------------------------------------------
rem File Name : mqfixinst.cmd
rem Descriptive File Name : Install an interim or test fix
rem ---------------------------------------------------------------------------
rem   <copyright
rem   notice="lm-source-program"
rem   pids="5724-H72"
rem   years="2012,2014"
rem   crc="2236575084" >
rem   Licensed Materials - Property of IBM
rem
rem   5724-H72
rem
rem   (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved.
rem
rem   US Government Users Restricted Rights - Use, duplication or
rem   disclosure restricted by GSA ADP Schedule Contract with
rem   IBM Corp.
rem   </copyright>
rem ---------------------------------------------------------------------------
rem Version 1.0
rem %Z% %W%  %I% %E% %U%
rem ---------------------------------------------------------------------------
rem File Description :
rem
rem This script is used to install or remove an interim or test fix provided by
rem IBM support into a given installation that you name on the command line.
rem
rem Warning: This script is provided to install the fix you have received from
rem          IBM.  It might be changed in future, so please do not store it and
rem          re-use for another fix in future.  Please instead use the latest
rem          script that IBM includes with the later fix.
rem
rem Parameters:
rem   <Product_Installation_Dir> {ACTION} FixName
rem
rem Where ACTION is one of
rem   testinstall   - Log to the console the actions it would perform when
rem                   installing the fix, but make no actual changes
rem   testuninstall - Log to the screen the actions it would perform when
rem                   uninstalling the fix, but make no actual changes
rem   install       - Install the interim / test fix
rem   uninstall     - Uninstall the interim / test fix
rem   uninstall_no_mqpatch_check - Uninstall avoiding the mqpatch check
rem
rem ---------------------------------------------------------------------------

setlocal
set MQF_RC=0
set MQF_FIXLOCATION="%~dp0"
set MQF_SCRIPTNAME=%~nx0
set MQF_NOPATCHCHECK=0
set MQF_ARGFIXID=Unsupplied

rem
rem Validate and parse the parameters supplied by the user
rem
call :ValidateParameters %1 %2 %3 %4
if not '%MQF_RC%'=='0' goto :Return

rem
rem Validate environment for this fix
rem
call :ValidateEnvironment
if not '%MQF_RC%'=='0' goto :Return

rem
rem Do the install or uninstall as appropriate
rem
if '%MQF_MQACTION%'=='INSTALL' call :DoInstall
if '%MQF_MQACTION%'=='UNINSTALL' call :DoUninstall
if not '%MQF_RC%'=='0' goto :Return

rem
rem We are done!
rem
call :LogEcho .Completed successfully...
goto :Return


rem ===========================================================================
rem Validate the parameters are of a valid syntax
rem ===========================================================================
:ValidateParameters

  rem ---------------------------------------------------------------------------
  rem Validate %1 is the product installation directory (quoted if containing
  rem spaces)
  rem ---------------------------------------------------------------------------
  if '%1'=='' (
    call :LogEcho .FAILURE - No product installation directory provided.
    set MQF_RC=2
    goto :DisplaySyntax
  )
  set MQF_MQINSTALLATIONDIR=%1

  rem Strip off any trailing \ if surrounded by quotes as it causes problems in
  rem findstr later on, and the odd syntax is because batch does not like
  rem unescaped, unmatching double quotes!
  if '^%MQF_MQINSTALLATIONDIR:~-2,1%^%MQF_MQINSTALLATIONDIR:~-1,1%'=='^\^"' (
    set MQF_MQINSTALLATIONDIR=%MQF_MQINSTALLATIONDIR:~0,-2%"
  )

  rem ---------------------------------------------------------------------------
  rem Calculate if this installation is registered with the O/S
  rem ---------------------------------------------------------------------------
  set MQF_REGISTERED=Y
  set MQF_71ORHIGHER=Y
  set MQF_PRIMARY=N
  set MQF_MQLIGHT=N
  set MQF_MQROOTDIR=%MQF_MQINSTALLATIONDIR%
  set MQF_MQBINDIR=%MQF_MQINSTALLATIONDIR%\bin
  rem ---------------------------------------------------------------------------
  rem From MQ V8 we'll want to source our programs from the bin64 dir.  But V7.5
  rem also has a bin64 dir.  We use this heuristic: if amqicvar is in the bin64
  rem then this is MQ V8.
  rem ---------------------------------------------------------------------------
  if exist %MQF_MQINSTALLATIONDIR%\bin64\amqicvar.exe (
    set MQF_MQBINDIR=%MQF_MQINSTALLATIONDIR%\bin64
  )
  if exist %MQF_MQINSTALLATIONDIR%\iso-swid\*IBM_MQ_Light*.swidtag (
    set MQF_REGISTERED=N
    set MQF_MQLIGHT=Y
    set MQF_MQROOTDIR=%MQF_MQINSTALLATIONDIR%\runtime
    set MQF_MQBINDIR=%MQF_MQINSTALLATIONDIR%\runtime\bin64
    set MQF_LNAME=IBM MQ Light
    set MQF_SNAME=MQ Light
  ) else (
    set MQF_LNAME=WebSphere MQ
    set MQF_SNAME=MQ
  )

  call :LogEcho .--------------------------
  call :LogEcho .%MQF_LNAME% Fix Installer
  call :LogEcho .--------------------------

  rem ---------------------------------------------------------------------------
  rem Check that the product installation directory exists
  rem ---------------------------------------------------------------------------
  if not exist %MQF_MQINSTALLATIONDIR% (
    call :LogEcho .FAILURE - Invalid %MQF_SNAME% installation directory provided - If it contains a space
    call :LogEcho .        remember to put double quotes around it
    set MQF_RC=4
    goto :DisplaySyntax
  )
  if not exist %MQF_MQBINDIR%\amqxcs2.dll (
    call :LogEcho .FAILURE - %MQF_SNAME% installation directory is not a valid %MQF_SNAME% installation.
    set MQF_RC=6
    goto :DisplaySyntax
  )

  rem ---------------------------------------------------------------------------
  rem Validate %2 is one of the valid options
  rem ---------------------------------------------------------------------------
  set MQF_MQTEST=?
  set MQF_MQACTION=UNKNOWN

  if '%2'=='' (
    call :LogEcho .FAILURE - No action supplied. Please give one of TESTINSTALL, TESTUNINSTALL,
    call :LogEcho .        INSTALL, UNINSTALL or UNINSTALL_NO_MQPATCH_CHECK
    set MQF_RC=8
    goto :DisplaySyntax
  ) else if /I '%2'=='TESTINSTALL' (
    set MQF_MQTEST=Y
    set MQF_MQACTION=INSTALL
  ) else if /I '%2'=='TESTUNINSTALL' (
    set MQF_MQTEST=Y
    set MQF_MQACTION=UNINSTALL
  ) else if /I '%2'=='INSTALL' (
    set MQF_MQTEST=N
    set MQF_MQACTION=INSTALL
  ) else if /I '%2'=='UNINSTALL' (
    set MQF_MQTEST=N
    set MQF_MQACTION=UNINSTALL
  ) else if /I '%2'=='UNINSTALL_NO_MQPATCH_CHECK' (
    set MQF_MQTEST=N
    set MQF_MQACTION=UNINSTALL
    set MQF_NOPATCHCHECK=1
  ) else (
    call :LogEcho .FAILURE - Invalid action supplied. Please give one of TESTINSTALL,
    call :LogEcho .   TESTUNINSTALL, INSTALL, UNINSTALL or UNINSTALL_NO_MQPATCH_CHECK
    set MQF_RC=10
    goto :DisplaySyntax
  )

  rem ---------------------------------------------------------------------------
  rem Validate the fix id was supplied
  rem ---------------------------------------------------------------------------
  if '%3'=='' (
    call :LogEcho .FAILURE - Failed to supply fix name argument
    set MQF_RC=11
    goto :DisplaySyntax
  ) else (
    set MQF_ARGFIXID=%3
  )
  rem Remove any surrounding quotes
  set MQF_ARGFIXID=%MQF_ARGFIXID:"=%


  rem ---------------------------------------------------------------------------
  rem Validate no more parameters are supplied
  rem ---------------------------------------------------------------------------
  if not '%4'=='' (
    call :LogEcho .FAILURE - Invalid number of parameters.
    set MQF_RC=12
    goto :DisplaySyntax
  )

  rem ---------------------------------------------------------------------------
  rem Finished, so we now can enable logging if we are not dry-run
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='N' set MQF_LOGECHOFILE=Y
  goto :EOF

:DisplaySyntax
  call :LogEcho .Syntax:
  call :LogEcho .  %MQF_SCRIPTNAME% {product install directory} {action} FixId
  call :LogEcho .       where action is one of INSTALL, TESTINSTALL,
  call :LogEcho .         UNINSTALL, TESTUNINSTALL or UNINSTALL_NO_MQPATCH_CHECK
  goto :EOF


rem ===========================================================================
rem Validate the environment matches what the fix is supposed to be installed
rem on and that the user is an administrator (unless in test mode)
rem ===========================================================================
:ValidateEnvironment

  rem ---------------------------------------------------------------------------
  rem Confirm the user is an Administrator - Unless in test mode
  rem ---------------------------------------------------------------------------
  if '%MQF_REGISTERED%'=='Y' (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Skipping check that the user is an administrator / running elevated.
      goto :skipAdminCheck
    )

    net session >NUL 2>&1
    if ERRORLEVEL 1 (
      call :LogEcho .FAILURE - Not running elevated / as an Administrator.
      set MQF_RC=20
      goto :EOF
    )
  )
  :skipAdminCheck

  rem ---------------------------------------------------------------------------
  rem Change to the location the script is running from, and then ensure it
  rem contains a README.txt (used during install and uninstall to get the
  rem fix details)
  rem ---------------------------------------------------------------------------
  cd /d %MQF_FIXLOCATION%
  if not exist ".\README.txt" (
    call :LogEcho .FAILURE - Current directory does not contain fix readme.
    set MQF_RC=22
    goto :EOF
  )
  if not exist ".\FIX.dat" (
    call :LogEcho .FAILURE - Current directory does not contain fix data file.
    set MQF_RC=23
    goto :EOF
  )

  rem ---------------------------------------------------------------------------
  rem Extract fix name and intended VRMF from fix.dat
  rem ---------------------------------------------------------------------------
  echo Calculating the fix id and intended maintenance levels from fix.dat
  findstr /c:"Fix name:" FIX.dat >%TEMP%\MQFIXINSTALL.TMP 2>&1
  rem Strip spaces from second token
  for /f "usebackq tokens=1-6* delims=:- " %%i in (%TEMP%\MQFIXINSTALL.TMP) do (
    set MQF_SHORTFIXID=%%o
  )
  for /f "usebackq tokens=1,2* delims=: " %%i in (%TEMP%\MQFIXINSTALL.TMP) do (
    set MQF_FIXID=%%k
  )
  findstr /c:"Maintenance level:" FIX.dat >%TEMP%\MQFIXINSTALL.TMP 2>&1
  rem Strip spaces from second token
  for /f "usebackq tokens=1,2 delims=:" %%i in (%TEMP%\MQFIXINSTALL.TMP) do (
    for /f %%k in ("%%j") do set MQF_FIXVRMF=%%k
  )

  rem ---------------------------------------------------------------------------
  rem Confirm the fixid matches what the user supplied
  rem ---------------------------------------------------------------------------
  if /I not "%MQF_ARGFIXID%"=="%MQF_FIXID%" (
    call :LogEcho .FAILURE: Fix name validity check failed. Are you sure this is the
    call :LogEcho .   fix you want to install/uninstall?
    call :LogEcho .   Supplied fixid is %MQF_ARGFIXID%
    call :LogEcho .   Actual fixid is   %MQF_FIXID%
    del %TEMP%\MQFIXINSTALL.TMP
    set MQF_RC=25
    goto :EOF
  )

  rem ---------------------------------------------------------------------------
  rem Confirm the fix is being installed to the correct VRMF
  rem ---------------------------------------------------------------------------
  if '%MQF_MQLIGHT%'=='Y' (
    %MQF_MQBINDIR%\mqltver.exe -f 2 -b >%TEMP%\MQFIXINSTALL.TMP 2>&1
  ) else (
    %MQF_MQBINDIR%\dspmqver.exe -f 2 -b >%TEMP%\MQFIXINSTALL.TMP 2>&1
  )
  for /F "usebackq" %%i in ("%TEMP%\MQFIXINSTALL.TMP") do set MQF_THISVRMF=%%i
  rem Workaround for MQ Light version <= 1.0.0.1 where mqltver always returns all fields
  if '%MQF_THISVRMF%'=='LicenseType:' (
    for /F "usebackq tokens=1,2* delims=: " %%i in ("%TEMP%\MQFIXINSTALL.TMP") do (
      if "%%i"=="Version" set MQF_THISVRMF=%%j
    )
  )
  if not '%MQF_THISVRMF%'=='%MQF_FIXVRMF%' (
    call :LogEcho .FAILURE - Fix is intended for %MQF_FIXVRMF% but the %MQF_SNAME% install is %MQF_THISVRMF%
    del %TEMP%\MQFIXINSTALL.TMP
    set MQF_RC=26
    goto :EOF
  )
  del %TEMP%\MQFIXINSTALL.TMP

  rem ---------------------------------------------------------------------------
  rem Check mqpatch.dat for evidence that the fix is already installed
  rem
  rem For install:
  rem If the fix is already installed, we write only a warning and then
  rem exit with 0.
  rem
  rem For uninstall:
  rem If the fix is NOT already installed, we write only a warning and then
  rem exit with 0.
  rem ---------------------------------------------------------------------------
  call :LogEcho .Checking mqpatch.dat for evidence of %MQF_SHORTFIXID%
  findstr /I %MQF_SHORTFIXID% %MQF_MQROOTDIR%\mqpatch.dat 2>NUL

  rem If found in the file, errorlevel is 0. If file does not exist or not found
  rem then error level is non zero (usually 1)
  if '%errorlevel%'=='0' (
    if '%MQF_MQACTION%'=='INSTALL' (
      call :LogEcho .WARNING: The fix is already installed. Exiting as success.
      set MQF_RC=-1
      goto :EOF
    ) else (
      call :LogEcho .     Found entry so fix is ok to be uninstalled.
    )
  ) else (
    if '%MQF_MQACTION%'=='INSTALL' (
      call :LogEcho .     Fix is not already installed so continuing
    ) else (
      if '%MQF_NOPATCHCHECK%'=='0' (
        call :LogEcho .WARNING: mqpatch.dat does not mention this fix.
        call :LogEcho .     Please investigate whether it really was installed
        call :LogEcho .     The updates to the mqpatch.dat file might have failed
        call :LogEcho .     This check on the mqpatch.dat file can be bypassed
        call :LogEcho .     by using the UNINSTALL_NO_MQPATCH_CHECK command.
        call :LogEcho .     Exiting as success.
        set MQF_RC=-1
        goto :EOF
      ) else (
        call :LogEcho .WARNING: mqpatch.dat does not mention this fix,
        call :LogEcho .         but we have been told to continue anyway.
      )
    )
  )

  if '%MQF_REGISTERED%=='Y' (
    rem -------------------------------------------------------------------------
    rem Calculate if 7.1 or higher
    rem -------------------------------------------------------------------------
    for /F "tokens=1,2,3,4 delims=." %%i in ("%MQF_FIXVRMF%") do (
      if %%i lss 7 (
        set MQF_71ORHIGHER=N
      ) else if %%i equ 7 (
        if %%j EQU 0 (
          set MQF_71ORHIGHER=N
        )
      )
    )

    rem -------------------------------------------------------------------------
    rem Calculate if primary or not
    rem -------------------------------------------------------------------------
    if '%MQF_71ORHIGHER%'=='N' (
      set MQF_PRIMARY=Y
    ) else (
      %MQF_MQBINDIR%\dspmqver.exe -f 2048 -b >%TEMP%\MQFIXINSTALL.TMP 2>&1
      for /F "usebackq" %%i in ("%TEMP%\MQFIXINSTALL.TMP") do (
        if /I '%%i'=='Yes' set MQF_PRIMARY=Y
      )
    )
  )

  if '%MQF_71ORHIGHER%'=='N' (
    set MQF_SERVICENAME=IBM MQSeries
    set MQF_INSTALLATIONNAME=this installation
  ) else (
    %MQF_MQBINDIR%\dspmqver.exe -f 512 -b >%TEMP%\MQFIXINSTALL.TMP 2>&1
    for /F "usebackq" %%i in ("%TEMP%\MQFIXINSTALL.TMP") do (
      set MQF_SERVICENAME=IBM WebSphere MQ ^(%%i^)
      set MQF_INSTALLATIONNAME=%%i
    )
  )

  rem ---------------------------------------------------------------------------
  rem Confirm the service is stopped
  rem We do this by 'net start' which lists all running services, and then search
  rem in it for the service of the installation we are replacing. If it is in the
  rem list, it is not stopped.
  rem ---------------------------------------------------------------------------
  if '%MQF_REGISTERED%'=='Y' (

    net start > %TEMP%\MQFIXINSTALL.TMP
    findstr /C:"%MQF_SERVICENAME%" %TEMP%\MQFIXINSTALL.TMP >nul 2>&1

    if errorlevel 1 (
      call :LogEcho .%MQF_SNAME% Status:      %MQF_SNAME% Service for %MQF_INSTALLATIONNAME% is offline.
    ) else (
      call :LogEcho .FAILURE - %MQF_SNAME% Service for %MQF_INSTALLATIONNAME% is still running, please end
      call :LogEcho .          it using endmqsvc, plus all %MQF_SNAME% applications using this install.
      set MQF_RC=27
      del %TEMP%\MQFIXINSTALL.TMP >nul 2>&1
      goto :EOF
    )
    del %TEMP%\MQFIXINSTALL.TMP >nul 2>&1
  )

  rem ---------------------------------------------------------------------------
  rem See if there are any connected applications and if so, list them.
  rem We do this by 'dspmq -a' and if that returns a non-zero return code then
  rem there is an application (non-java) which is still running.
  rem ---------------------------------------------------------------------------
  if '%MQF_71ORHIGHER%'=='Y' (
    if exist %MQF_MQBINDIR%\dspmq.exe (
      %MQF_MQBINDIR%\dspmq.exe -a >%TEMP%\MQFIXINSTALL.TMP 2>&1

      if errorlevel 1 (
        call :LogEcho .Appl Status:         **FAILED**
        call :LogEcho .FAILURE - An %MQF_SNAME% application is still using binaries from this install.
        if '%MQF_MQLIGHT%'=='Y' (
          call :LogEcho .          Please stop %MQF_SNAME% using mqlight-stop.
        ) else (
          call :LogEcho .          Please ensure all %MQF_SNAME% applications are ended.
        )
        call :LogEcho .          Running process ids [PIDS] listed below if known:

        %MQF_MQBINDIR%\dspmq.exe -c >%TEMP%\MQFIXINSTALL.TMP 2>&1
        findstr /C:"PID(" %TEMP%\MQFIXINSTALL.TMP >%TEMP%\MQFIXINSTALL.TMP2 2>&1

        for /F "usebackq tokens=1*" %%i in ("%TEMP%\MQFIXINSTALL.TMP2") do (
          call :LogEcho .             %%i %%j
        )

        set MQF_RC=28
        del %TEMP%\MQFIXINSTALL.TMP >nul 2>&1
        goto :EOF
      ) else (
        call :LogEcho .Appl Status:         All native applications disconnected
      )
    ) else (
      call :LogEcho .Appl Status:         dspmq.exe not found - client only install
    )
  )

  rem ---------------------------------------------------------------------------
  rem Log some key details
  rem ---------------------------------------------------------------------------
  call :LogEcho .Date:                %DATE%
  call :LogEcho .Time:                %TIME%
  call :LogEcho .FixId:               %MQF_FIXID%
  call :LogEcho .VRMF:                %MQF_FIXVRMF%  (7.1 or higher? %MQF_71ORHIGHER%)
  call :LogEcho .Fix Source:          %MQF_FIXLOCATION%
  if '%MQF_MQTEST%'=='Y' call :LogEcho .Action:              %MQF_MQACTION% - Testing only
  if '%MQF_MQTEST%'=='N' call :LogEcho .Action:              %MQF_MQACTION%
  call :LogEcho .Install Directory:   %MQF_MQINSTALLATIONDIR%
  for /F "tokens=1*" %%i in ('ver') do call :LogEcho .OS:                  %%i %%j
  call :LogEcho .Installation:        %MQF_INSTALLATIONNAME% (Primary? %MQF_PRIMARY%  Registered? %MQF_REGISTERED%)
  if not '%MQF_MQROOTDIR%'=='%MQF_MQINSTALLATIONDIR%' call :LogEcho .Root Dir:            %MQF_MQROOTDIR%

  rem ---------------------------------------------------------------------------
  rem Finished
  rem ---------------------------------------------------------------------------
  goto :EOF


rem ===========================================================================
rem Perform the installation steps (or display them if in test mode)
rem ===========================================================================
:DoInstall

  call :LogEcho .-----------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .Installing the fix - Dry run / test mode only
  ) else (
    call :LogEcho .Installing the fix
  )
  call :LogEcho .-----------------------------------------------

  rem ---------------------------------------------------------------------------
  rem Ensure we have a ship directory
  rem ---------------------------------------------------------------------------
  if not exist "ship\*.*" (
    call :LogEcho .FAILURE - ship directory missing or empty.
    set MQF_RC=24
    goto :EOF
  )

  rem ---------------------------------------------------------------------------
  rem Create the backup directory and warn if one already exists. It is valid to
  rem install multiple fixes but it helps to be aware.
  rem ---------------------------------------------------------------------------
  set MQF_INSTALLBACKUP=%MQF_MQINSTALLATIONDIR%\Fix-Backups.%MQF_FIXVRMF%
  set MQF_INSTFIXBACKUP=%MQF_INSTALLBACKUP%\%MQF_FIXID%

  if exist %MQF_INSTALLBACKUP% (
    call :LogEcho .WARNING: Backup directory already exists - another fix may be installed
  ) else (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_INSTALLBACKUP%
    ) else (
      call :LogEcho .Creating backup directory %MQF_INSTALLBACKUP%
      MD %MQF_INSTALLBACKUP%
    )
  )

  if not exist %MQF_INSTFIXBACKUP% (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_INSTFIXBACKUP%
    ) else (
      call :LogEcho .Creating backup directory %MQF_INSTFIXBACKUP%
      MD %MQF_INSTFIXBACKUP%
    )
  )

  if not exist %MQF_INSTFIXBACKUP%\files (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_INSTFIXBACKUP%\files
    ) else (
      call :LogEcho .Creating backup directory %MQF_INSTFIXBACKUP%\files
      MD %MQF_INSTFIXBACKUP%\files
    )
  )

  rem ---------------------------------------------------------------------------
  rem Check for the existence of a backed up file already there
  rem ---------------------------------------------------------------------------
  cd Ship
  call :LogEcho .Checking for potential conflicting fixes
  for /R %%i in (*.*) do (
     call :CheckForConflicts %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndInstall

  rem ---------------------------------------------------------------------------
  rem Save directory listing before we start (just in case!)
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP: Would have created a directory listing in %MQF_INSTFIXBACKUP%\beforeInstall.log
    call :LogEcho .SKIP:   by appending the command: dir /s %MQF_MQINSTALLATIONDIR%
  ) else (
    call :LogEcho .Creating directory listing to record pre-install state.
    echo %DATE% %TIME% ------------ Prior to install -------------- >> %MQF_INSTFIXBACKUP%\beforeInstall.log
    dir /s %MQF_MQINSTALLATIONDIR% >> %MQF_INSTFIXBACKUP%\beforeInstall.log
  )

  rem ---------------------------------------------------------------------------
  rem Take backups of the live files
  rem ---------------------------------------------------------------------------
  call :LogEcho .Backing up existing files
  set MQF_BACKUPTO=%MQF_INSTFIXBACKUP%\files
  cd Ship
  for /R %%i in (*.*) do (
     call :TakeBackup %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndInstall

  rem ---------------------------------------------------------------------------
  rem Copy this script into the backup directory
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP - Would have copied %MQF_SCRIPTNAME% into fix backup directory
    call :LogEcho .       By copying %MQF_FIXLOCATION%%MQF_SCRIPTNAME%
    call :LogEcho .               to %MQF_INSTFIXBACKUP%\%MQF_SCRIPTNAME%
    call :LogEcho .SKIP - Would have copied README.txt into fix backup directory
    call :LogEcho .       By copying %MQF_FIXLOCATION%README.txt
    call :LogEcho .               to %MQF_INSTFIXBACKUP%\README.txt
    call :LogEcho .SKIP - Would have copied FIX.dat into fix backup directory
    call :LogEcho .       By copying %MQF_FIXLOCATION%FIX.dat
    call :LogEcho .               to %MQF_INSTFIXBACKUP%\FIX.dat
  ) else (
    call :LogEcho .Copying %MQF_SCRIPTNAME% into fix backup directory
    copy %MQF_FIXLOCATION%%MQF_SCRIPTNAME% %MQF_INSTFIXBACKUP%\%MQF_SCRIPTNAME% /y
    call :LogEcho .Copying README.txt into fix backup directory
    copy %MQF_FIXLOCATION%README.txt %MQF_INSTFIXBACKUP%\README.txt /y
    call :LogEcho .Copying FIX.dat into fix backup directory
    copy %MQF_FIXLOCATION%FIX.dat %MQF_INSTFIXBACKUP%\FIX.dat /y
  )

  rem ---------------------------------------------------------------------------
  rem If 7.1 or higher, and file is a .exe.config, run amqicvar against it.
  rem ---------------------------------------------------------------------------
  if "%MQF_REGISTERED%"=="Y" (
    if "%MQF_71ORHIGHER%"=="Y" (
      call :LogEcho .Making pre-install time modifications to files if appropriate
      for /R %%i in (*.*) do (
       call :CorrectPaths %%i
      )
      if not '%MQF_RC%'=='0' goto :EndInstall
    )
  )

  rem ---------------------------------------------------------------------------
  rem Remove the files from the live location
  rem ---------------------------------------------------------------------------
  call :LogEcho .Removing existing files
  cd Ship
  for /R %%i in (*.*) do (
     call :RemoveLiveCopy %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndInstall

  rem ---------------------------------------------------------------------------
  rem Copy in the replacement files
  rem ---------------------------------------------------------------------------
  call :LogEcho .Installing replacement files
  set MQF_COPYFROM=%MQF_FIXLOCATION%ship\
  cd Ship
  for /R %%i in (*.*) do (
     call :ReplaceLiveCopy %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndInstall

  rem ---------------------------------------------------------------------------
  rem Fix up the GAC (In case .NET parts involved)
  rem ---------------------------------------------------------------------------
  if "%MQF_REGISTERED%"=="Y" (
    call :LogEcho .Reregistering any .NET components
    call :DoReRegister
    if not '%MQF_RC%'=='0' goto :EndInstall
  )

  rem ---------------------------------------------------------------------------
  rem Save directory listing after we complete (just in case!)
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP: Would have created a directory listing in %MQF_INSTFIXBACKUP%\afterInstall.log
    call :LogEcho .SKIP:   by appending the command: dir /s %MQF_MQINSTALLATIONDIR%
  ) else (
    call :LogEcho .Creating directory listing to record post-install state.
    echo %DATE% %TIME% ------------ Post install -------------- >> %MQF_INSTFIXBACKUP%\afterInstall.log
    dir /s %MQF_MQINSTALLATIONDIR% >> %MQF_INSTFIXBACKUP%\afterInstall.log
  )

  rem ---------------------------------------------------------------------------
  rem Add info to the product wide patch file
  rem ---------------------------------------------------------------------------
  call :LogEcho .Adding info to product installed patches
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .  SKIP: Would have backed up mqpatch.dat, then appended %MQF_SHORTFIXID% to it.
  ) else (
    if exist %MQF_MQROOTDIR%\mqpatch.dat (
      copy %MQF_MQROOTDIR%\mqpatch.dat %MQF_MQROOTDIR%\mqpatch.dat_bak /y >nul 2>&1
      FINDSTR /V /I /C:"%MQF_SHORTFIXID%" %MQF_MQROOTDIR%\mqpatch.dat_bak > %MQF_MQROOTDIR%\mqpatch.dat
    )
    echo %MQF_SHORTFIXID% # %date% %time% >> %MQF_MQROOTDIR%\mqpatch.dat
  )

  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .Test install / dry-run completed successfully.
    call :LogEcho .  Rerun this script again with the INSTALL option to actually perform the install.
  ) else (
    call :LogEcho .Fix installed successfully.
    if '%MQF_REGISTERED%'=='Y' (
      call :LogEcho .   Remember to restart the %MQF_SNAME% service, i.e. %MQF_MQBINDIR%\strmqsvc.exe
    )
  )

:EndInstall
  rem -------------------------------------------------------------------------
  rem Clean up any edited .config.exe's as we dont need them anymore
  rem Note: Do this quietly on error so the last line is the failure message
  rem -------------------------------------------------------------------------
  if '%MQF_RC%'=='0' (
    call :LogEcho . Cleaning up any temporary files
    del *_edited.* /s 2>nul
  ) else (
    del *_edited.* /s 2>nul >nul
  )
  goto :EOF


rem ===========================================================================
rem Check for one fix overwriting another
rem ===========================================================================
:CheckForConflicts
  if not '%MQF_RC%'=='0' goto :EOF
  set MQF_FILENAME=%*
  call :StripCD

  rem -------------------------------------------------------------------------
  rem Ignore the packaging files themselves
  rem -------------------------------------------------------------------------
  call :IsSkippable %MQF_FILENAME%
  if '%MQF_SKIP%'=='Y' goto :EOF

  rem -------------------------------------------------------------------------
  rem Look through all the directories and see if there is a match
  rem   but exclude our own fixid to allow recovery following a failure
  rem -------------------------------------------------------------------------
  for /d %%j in (%MQF_INSTALLBACKUP%\*) do (
    call :LogEcho .    Checking for %MQF_FILENAME% in backup directory %%j
    if exist %%j\files\%MQF_FILENAME% (
      if /I '%%~nxj'=='%MQF_FIXID%' (
        call :LogEcho .      Ignoring this one as just reapplying this fix.
      ) else (
        rem -------------------------------------------------------------------------
        rem If we found a match, we have a conflicting patch installed
        rem -------------------------------------------------------------------------
        call :LogEcho .FAILURE - Conflicting fixes. %MQF_FILENAME% already patched
        call :LogEcho .  %%~nxj cannot coexist with this fix.
        call :LogEcho .        Contact IBM support for merged fix.
        set MQF_RC=40
      )
    )
  )

  goto :EOF

rem ===========================================================================
rem Take a backup copy of the live file
rem ===========================================================================
:TakeBackup
  if not '%MQF_RC%'=='0' goto :EOF
  set MQF_FILENAME=%*
  call :StripCD

  rem -------------------------------------------------------------------------
  rem Ignore the packaging files themselves
  rem -------------------------------------------------------------------------
  call :IsSkippable %MQF_FILENAME%
  if '%MQF_SKIP%'=='Y' goto :EOF

  rem -------------------------------------------------------------------------
  rem If backup already exists - fix is being reapplied / reuninstalled,
  rem that's fine.
  rem -------------------------------------------------------------------------
  if exist %MQF_BACKUPTO%\%MQF_FILENAME% (
    call :LogEcho .%MQF_FILENAME% already backed up - skipping need to take a backup
    goto :EOF
  )

  rem -------------------------------------------------------------------------
  rem We have a problem... To avoid conflicting names we need to put the copy
  rem into a subdirectory, but windows cannot create the directory during the
  rem copy so we have to create it here
  rem -------------------------------------------------------------------------
  set MQF_TEMP=%MQF_FILENAME%
  set MQF_SUBDIR=\
  :CreateSubDirs
  for /F "tokens=1* delims=\" %%j in ("%MQF_TEMP%") do (
    if not '%%k'=='' (
      if not exist %MQF_BACKUPTO%%MQF_SUBDIR%%%j (
        if '%MQF_MQTEST%'=='Y' (
          call :LogEcho .  SKIP: Would have created directory %MQF_BACKUPTO%%MQF_SUBDIR%%%j
        ) else (
          call :LogEcho .  Creating directory %MQF_BACKUPTO%%MQF_SUBDIR%%%j
          md %MQF_BACKUPTO%%MQF_SUBDIR%%%j
        )
      )
      if "%MQF_SUBDIR%"=="\" (
        set MQF_SUBDIR=\%%j\
      ) else (
        set MQF_SUBDIR=%MQF_SUBDIR%%%j\
      )
    )
    set MQF_TEMP=%%k
  )
  if not '%MQF_TEMP%'=='' goto :CreateSubDirs

  rem -------------------------------------------------------------------------
  rem Now take the copy
  rem -------------------------------------------------------------------------
  if exist %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME% (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .  SKIP: Would have copied %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
      call :LogEcho .  SKIP:                to %MQF_BACKUPTO%\%MQF_FILENAME%
    ) else (
      call :LogEcho .  Copying %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
      call :LogEcho .       to %MQF_BACKUPTO%\%MQF_FILENAME%
      copy /B /V %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME% %MQF_BACKUPTO%\%MQF_FILENAME%
      if ERRORLEVEL 1 (
        call :LogEcho .  FAILURE - Failed to backup live file %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
        call :LogEcho .           to %MQF_BACKUPTO%\%MQF_FILENAME%
        set MQF_RC=42
      )
    )
  ) else (
    call :LogEcho .  File %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME% does not exist - nothing to back up.
  )
  goto :EOF

rem ===========================================================================
rem Correct the paths if necessarily - usually performed by installer
rem ===========================================================================
:CorrectPaths
  if not '%MQF_RC%'=='0' goto :EOF
  set MQF_FILENAME=%*
  call :StripCD

  rem -------------------------------------------------------------------------
  rem Ignore the packaging files themselves
  rem -------------------------------------------------------------------------
  call :IsSkippable %MQF_FILENAME%
  if '%MQF_SKIP%'=='Y' goto :EOF

  for %%j in (%MQF_FILENAME%) do (
    if /I '%%~xj'=='.config' (
      rem -------------------------------------------------------------------------
      rem The following command queries whether a replacement is necessary,
      rem   returning 0 if it is
      rem -------------------------------------------------------------------------
      call :LogEcho .  Checking if file %MQF_FILENAME% needs the installation path fixing up
      %MQF_MQBINDIR%\amqicvar.exe -q -t -w -f %MQF_FILENAME% -i MQ_INSTALLATION_PATH=%MQF_MQROOTDIR% >NUL 2>&1
      if errorlevel 1 (
        call :LogEcho .  ..no alterations needed
      ) else (
        if '%MQF_MQTEST%'=='Y' (
          call :LogEcho .  SKIP: Would have backed up the config file %MQF_FILENAME%
          call :LogEcho .                                          to %MQF_FILENAME%_edited
          call :LogEcho .  SKIP: Would have tried to correct the path by running:
          call :LogEcho .  SKIP:   %MQF_MQBINDIR%\amqicvar.exe -t -w -f %MQF_FILENAME%_edited -i MQ_INSTALLATION_PATH=%MQF_MQROOTDIR%

          rem -------------------------------------------------------------------------
          rem Although running in test mode, we need the _edited file to exist in order
          rem for the subsequent code to display the accurate steps it would have done.
          rem Create a dummy temporary file - this is in the fix directory not the MQ
          rem installation directory, ie the dry-run nature will not touch the MQ tree.
          rem -------------------------------------------------------------------------
          echo DUMMY > %MQF_FILENAME%_edited

        ) else (
          call :LogEcho .    Correcting the installation path in %%~nxj
          call :LogEcho .      Copying it to %MQF_FILENAME%_edited
          copy %MQF_FILENAME% %MQF_FILENAME%_edited /y
          if errorlevel 1 (
            call :LogEcho .    FAILURE - amqicvar failed to copy the config file %%~nxj to edit it
            set MQF_RC=44
          )

          call :LogEcho .      Correcting the path with the config file
          %MQF_MQBINDIR%\amqicvar.exe -t -w -f %MQF_FILENAME%_edited -i MQ_INSTALLATION_PATH=%MQF_MQROOTDIR%
          if errorlevel 1 (
            call :LogEcho .    FAILURE - amqicvar failed to correct the paths in the file %%~nxj
            call :LogEcho .              before installing it. Perhaps read only?
            set MQF_RC=46
          )
        )
      )
    )
  )
  goto :EOF

rem ===========================================================================
rem Remove the live copy of the files we need to replace
rem ===========================================================================
:RemoveLiveCopy
  if not '%MQF_RC%'=='0' goto :EOF
  set MQF_FILENAME=%*
  call :StripCD

  rem -------------------------------------------------------------------------
  rem Ignore the packaging files themselves
  rem -------------------------------------------------------------------------
  call :IsSkippable %MQF_FILENAME%
  if '%MQF_SKIP%'=='Y' goto :EOF

  if exist %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME% (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .  SKIP: Would have tried to attrib -r %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
      call :LogEcho .  SKIP: Would have tried to delete %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
    ) else (

      rem -------------------------------------------------------------------------
      rem Note: 'del' doesnt set errorlevel, so check afterwards
      rem -------------------------------------------------------------------------
      call :LogEcho .  Attempting to delete %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
      attrib -r %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
      del %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%

      if exist %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME% (
        call :LogEcho .    FAILURE - Failed to delete file %MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
        set MQF_RC=48
      )
    )
  )
  goto :EOF

rem ===========================================================================
rem Copy in the new files into the live tree
rem ===========================================================================
:ReplaceLiveCopy
  if not '%MQF_RC%'=='0' goto :EOF
  set MQF_FILENAME=%*
  call :StripCD

  rem -------------------------------------------------------------------------
  rem Ignore the packaging files themselves
  rem -------------------------------------------------------------------------
  call :IsSkippable %MQF_FILENAME%
  if '%MQF_SKIP%'=='Y' goto :EOF

  rem -------------------------------------------------------------------------
  rem If there is the same file which has been edited to correct the path then
  rem ignore this one as we will handle the other in a special way
  rem -------------------------------------------------------------------------
  if exist %MQF_FILENAME%_edited (
    call :LogEcho .  Ignoring %MQF_FILENAME% as we need to use the edited version
    goto :EOF
  )

  rem -------------------------------------------------------------------------
  rem If this is an edited file, special case it by renaming during the copy
  rem -------------------------------------------------------------------------
  if not '%MQF_FILENAME:_edited=%'=='%MQF_FILENAME%' (
    set MQF_SRC=%MQF_COPYFROM%%MQF_FILENAME%
    set MQF_DEST=%MQF_MQINSTALLATIONDIR%\%MQF_FILENAME:_edited=%
  ) else (
    set MQF_SRC=%MQF_COPYFROM%%MQF_FILENAME%
    set MQF_DEST=%MQF_MQINSTALLATIONDIR%\%MQF_FILENAME%
  )

  rem -------------------------------------------------------------------------
  rem We have a problem... If we are including a new file in a new subdirectory
  rem windows cannot create the directory during the copy so we have to create it here
  rem -------------------------------------------------------------------------
  set MQF_TEMP=%MQF_FILENAME%
  set MQF_SUBDIR=
  :CreateSubDirsInstall
  for /F "tokens=1* delims=\" %%j in ("%MQF_TEMP%") do (
    if not '%%k'=='' (
      if not exist %MQF_MQINSTALLATIONDIR%%MQF_SUBDIR%\%%j (
        if '%MQF_MQTEST%'=='Y' (
          call :LogEcho .  SKIP: Would have created directory %MQF_MQINSTALLATIONDIR%%MQF_SUBDIR%\%%j
        ) else (
          call :LogEcho .  Creating directory %MQF_MQINSTALLATIONDIR%%MQF_SUBDIR%\%%j
          md %MQF_MQINSTALLATIONDIR%%MQF_SUBDIR%\%%j
        )
      )
      set MQF_SUBDIR=%MQF_SUBDIR%\%%j
    )
    set MQF_TEMP=%%k
  )
  if not '%MQF_TEMP%'=='' goto :CreateSubDirsInstall


  rem -------------------------------------------------------------------------
  rem Do the copy, and double check it has worked.
  rem -------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    if '%MQF_MQACTION%'=='INSTALL' (
      call :LogEcho .  SKIP: Would have copied %MQF_SRC%
    ) else (
      call :LogEcho .  SKIP: Would have moved  %MQF_SRC%
    )
    call :LogEcho .                   to     %MQF_DEST%
  ) else (

    if '%MQF_MQACTION%'=='INSTALL' (
      call :LogEcho .  Attempting to copy in %MQF_SRC%
      call :LogEcho .                     as %MQF_DEST%
      copy %MQF_SRC% %MQF_DEST% /b /v
      if errorlevel 1 (
        call :LogEcho .    FAILURE - Failed to copy in file %MQF_DEST%
        set MQF_RC=50
      )
    ) else (
      call :LogEcho .  Attempting to move %MQF_SRC%
      call :LogEcho .                  to %MQF_DEST%
      move %MQF_SRC% %MQF_DEST%
      if errorlevel 1 (
        call :LogEcho .    FAILURE - Failed to move in file %MQF_DEST%
        set MQF_RC=52
      )
    )

    if not exist %MQF_DEST% (
      call :LogEcho .    FAILURE - File %MQF_DEST% still missing
      set MQF_RC=54
    )
  )
  goto :EOF


rem ===========================================================================
rem Strip the current directory from a fully qualified filename
rem Caller sets MQF_FILNAME to the filename, we remove %CD% from it
rem ===========================================================================
:StripCD
  rem ---------------------------------------------------------------------------
  rem The filename we are passed is a full path name whereas we want it relative
  rem to where we are, so we can put it in the same place in the product
  rem installation directory. Windows doesnt have good string parsing, hence this
  rem odd code - strip character by character until the current directory has
  rem been removed.
  rem ---------------------------------------------------------------------------
  if not exist MQF_REMOVE set MQF_REMOVE=%CD%

  :addFileStripChar
  if '"%MQF_REMOVE%"'=='""' goto :addFileStrippedOK
  if not '"%MQF_REMOVE:~0,1%"'=='"%MQF_FILENAME:~0,1%"' goto :addFileStrippedFailed
  set MQF_FILENAME=%MQF_FILENAME:~1%
  set MQF_REMOVE=%MQF_REMOVE:~1%
  goto :addFileStripChar

  :addFileStrippedFailed
  call :LogEcho .FAILURE - Failed to build list of files - Contact IBM support
  set MQF_RC=60
  goto :EOF

  rem Remove preceeding \
  :addFileStrippedOK
  if not '%MQF_FILENAME:~0,1%'=='\' goto :addFileStrippedFailed
  set MQF_FILENAME=%MQF_FILENAME:~1%
  set MQF_REMOVE=
goto :EOF


rem ===========================================================================
rem Write to the log file and the screen a message
rem    Note: We still lose stdout/stderr of any command we execute
rem    Note: We strip off the first character as indenting spaces are lost in
rem          a batch call, so use a '.' to avoid the whitespace issue then
rem          remove it before issuing it to the screen / file.
rem ===========================================================================
:LogEcho
  set MQF_TEMPLINE=%*
  set MQF_TEMPLINE=%MQF_TEMPLINE:~1%
  if '%MQF_LOGECHOFILE%'=='Y' echo %DATE% %TIME% %MQF_TEMPLINE% >> %MQF_MQROOTDIR%\mqpatch.log
  echo %MQF_TEMPLINE%
  goto :EOF

rem ===========================================================================
rem Indicate whether to skip the file in question or not. When installing we
rem do not want to copy in mqfixinst.cmd nor README.txt/FIX.dat, and when
rem uninstalling we do not want to copy back mqfixinst.cmd, nor beforeInstall.log
rem or afterInstall.log
rem ===========================================================================
:IsSkippable
  set MQF_SKIP=N
  if '%MQF_MQACTION%'=='INSTALL' (
    if /I '%*'=='mqfixinst.cmd' set MQF_SKIP=Y
    if /I '%*'=='README.txt' set MQF_SKIP=Y
    if /I '%*'=='FIX.dat' set MQF_SKIP=Y
  )

  if '%MQF_MQACTION%'=='UNINSTALL' (
    if /I '%*'=='mqfixinst.cmd' set MQF_SKIP=Y
    if /I '%*'=='beforeInstall.log' set MQF_SKIP=Y
    if /I '%*'=='afterInstall.log' set MQF_SKIP=Y
    if /I '%*'=='README.txt' set MQF_SKIP=Y
    if /I '%*'=='FIX.dat' set MQF_SKIP=Y
  )

  if %MQF_SKIP%==Y call :LogEcho .   [Ignoring packaging related file %*]
  goto :EOF

rem ===========================================================================
rem Perform the uninstallation steps (or display them if in test mode)
rem ===========================================================================
:DoUninstall
  call :LogEcho .-----------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .Uninstalling the fix - Dry run / test mode only
  ) else (
    call :LogEcho .Uninstalling the fix
  )
  call :LogEcho .-----------------------------------------------

  rem ---------------------------------------------------------------------------
  rem Ensure the backup directory exists - error if it doesnt
  rem ---------------------------------------------------------------------------
  set MQF_INSTALLBACKUP=%MQF_MQINSTALLATIONDIR%\Fix-Backups.%MQF_FIXVRMF%
  set MQF_INSTFIXBACKUP=%MQF_MQINSTALLATIONDIR%\Fix-Backups.%MQF_FIXVRMF%\%MQF_FIXID%

  call :LogEcho .Checking there is a backup directory for this fix under %MQF_FIXVRMF%
  if not exist %MQF_INSTALLBACKUP% (
    call :LogEcho .FAILURE: Backup directory %MQF_INSTALLBACKUP% does not exist
    set MQF_RC=62
    goto :EOF
  )

  if not exist %MQF_INSTFIXBACKUP% (
    call :LogEcho .FAILURE: Backup directory %MQF_INSTFIXBACKUP% does not exist
    set MQF_RC=64
    goto :EOF
  )

  if not exist %MQF_INSTFIXBACKUP%\files\* (
    call :LogEcho .FAILURE: Backup directory %MQF_INSTFIXBACKUP% has no binaries
    set MQF_RC=65
    goto :EOF
  )

  rem ---------------------------------------------------------------------------
  rem Make an uninstall backup directory(!)
  rem We move the live files into the uninstall backup, then the restore the
  rem install backup into the live location. Whilst seriously overkill, would
  rem enable manual recovery should things go wrong mid-uninstall.
  rem ---------------------------------------------------------------------------
  set MQF_UNINSTALLBACKUP=%MQF_MQINSTALLATIONDIR%\Fix-Backups.%MQF_FIXVRMF%.uninstall
  set MQF_UNINSTFIXBACKUP=%MQF_UNINSTALLBACKUP%\%MQF_FIXID%

  call :LogEcho .Creating an uninstall backup directory (for recovery purposes)
  if exist %MQF_UNINSTALLBACKUP% (
    call :LogEcho .  Main uninstall backup directory already exists
  ) else (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_UNINSTALLBACKUP%
    ) else (
      call :LogEcho .Creating uninstall backup directory %MQF_UNINSTALLBACKUP%
      MD %MQF_UNINSTALLBACKUP%
    )
  )

  if not exist %MQF_UNINSTFIXBACKUP% (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_UNINSTFIXBACKUP%
    ) else (
      call :LogEcho .Creating backup directory %MQF_UNINSTFIXBACKUP%
      MD %MQF_UNINSTFIXBACKUP%
    )
  )

  if not exist %MQF_UNINSTFIXBACKUP%\files (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .SKIP: Would create %MQF_UNINSTFIXBACKUP%\files
    ) else (
      call :LogEcho .Creating backup directory %MQF_UNINSTFIXBACKUP%\files
      MD %MQF_UNINSTFIXBACKUP%\files
    )
  )

  rem ---------------------------------------------------------------------------
  rem Save directory listing before we start (just in case!)
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP: Would have created a directory listing in %MQF_UNINSTFIXBACKUP%\beforeInstall.log
    call :LogEcho .SKIP:   by appending the command: dir /s %MQF_MQINSTALLATIONDIR%
  ) else (
    call :LogEcho .Creating directory listing to record pre-uninstall state.
    echo %DATE% %TIME% ------------ Prior to uninstall -------------- >> %MQF_UNINSTFIXBACKUP%\beforeInstall.log
    dir /s %MQF_MQINSTALLATIONDIR% >> %MQF_UNINSTFIXBACKUP%\beforeInstall.log
  )

  rem ---------------------------------------------------------------------------
  rem Change to the backup directory which was taken when the fix was installed
  rem ---------------------------------------------------------------------------
  CD /D %MQF_INSTFIXBACKUP%

  rem ---------------------------------------------------------------------------
  rem Take backups of the live files
  rem ---------------------------------------------------------------------------
  call :LogEcho .Backing up existing live files
  set MQF_BACKUPTO=%MQF_UNINSTFIXBACKUP%\Files
  cd files
  for /R %%i in (*.*) do (
     call :TakeBackup %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndUnInstall

  rem ---------------------------------------------------------------------------
  rem Remove the files from the live location
  rem ---------------------------------------------------------------------------
  call :LogEcho .Removing existing files
  cd files
  for /R %%i in (*.*) do (
     call :RemoveLiveCopy %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndUnInstall

  rem ---------------------------------------------------------------------------
  rem Copy in the replacement files
  rem ---------------------------------------------------------------------------
  call :LogEcho .Installing replacement files
  set MQF_COPYFROM=%MQF_INSTFIXBACKUP%\files\
  cd files
  for /R %%i in (*.*) do (
     call :ReplaceLiveCopy %%i
  )
  cd ..
  if not '%MQF_RC%'=='0' goto :EndUnInstall

  rem ---------------------------------------------------------------------------
  rem Remove the readme.txt and fix.dat
  rem Note: We cannot remove the mqfixinst.cmd and we deliberately leave behind
  rem       the beforeInstall.log/afterinstall.log file (in case of a need for
  rem       problem diagnosis).
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP - Would have deleted %MQF_INSTFIXBACKUP%\README.txt
    call :LogEcho .SKIP - Would have deleted %MQF_INSTFIXBACKUP%\FIX.dat
  ) else (
    call :LogEcho .Deleting the %MQF_INSTFIXBACKUP%\README.txt
    del %MQF_INSTFIXBACKUP%\README.txt
    call :LogEcho .Deleting the %MQF_INSTFIXBACKUP%\FIX.dat
    del %MQF_INSTFIXBACKUP%\FIX.dat
  )

  rem ---------------------------------------------------------------------------
  rem Fix up the GAC (In case .NET parts involved)
  rem ---------------------------------------------------------------------------
  if '%MQF_REGISTERED%'=='Y' (
    call :LogEcho .Reregistering any .NET components
    call :DoReRegister
    if not '%MQF_RC%'=='0' goto :EndUnInstall
  )

  rem ---------------------------------------------------------------------------
  rem Save directory listing after we complete (just in case!)
  rem ---------------------------------------------------------------------------
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .SKIP: Would have created a directory listing in %MQF_UNINSTFIXBACKUP%\afterInstall.log
    call :LogEcho .SKIP:   by appending the command: dir /s %MQF_MQINSTALLATIONDIR%
  ) else (
    call :LogEcho .Creating directory listing to record post-uninstall state.
    echo %DATE% %TIME% ------------ Post uninstall -------------- >> %MQF_UNINSTFIXBACKUP%\afterInstall.log
    dir /s %MQF_MQINSTALLATIONDIR% >> %MQF_UNINSTFIXBACKUP%\afterInstall.log
  )

  rem ---------------------------------------------------------------------------
  rem Remove info from the product wide patch file
  rem   and if it reaches size 0, delete the file
  rem ---------------------------------------------------------------------------
  call :LogEcho .Removing info from the list of product installed patches
  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .  SKIP: Would have backed up mqpatch.dat, then removed %MQF_SHORTFIXID% from it.
  ) else (
    if exist %MQF_MQROOTDIR%\mqpatch.dat (
      copy %MQF_MQROOTDIR%\mqpatch.dat %MQF_MQROOTDIR%\mqpatch.dat_bak /y >nul 2>&1
      findstr /v /c:%MQF_SHORTFIXID% %MQF_MQROOTDIR%\mqpatch.dat_bak > %MQF_MQROOTDIR%\mqpatch.dat
    )
  )

  if '%MQF_MQTEST%'=='Y' (
    call :LogEcho .Test uninstall / dry-run completed successfully.
    call :LogEcho .  Rerun this script again with the UNINSTALL option to actually perform the uninstall.
  ) else (
    call :LogEcho .Fix uninstalled successfully.
    if '%MQF_REGISTERED%'=='Y' (
      call :LogEcho .   Remember to restart the %MQF_SNAME% service, i.e. %MQF_MQBINDIR%\strmqsvc.exe
    )
  )

:EndUninstall
  goto :EOF

rem ===========================================================================
rem Post install or rollback, we may need to set the GAC correctly.
rem
rem For pre-7.1, call amqiregisterdotnet
rem For post-7.1, call setmqinst if we are primary, otherwise nothing to do.
rem ===========================================================================
:DoReRegister
  if '%MQF_71ORHIGHER%'=='N' (
    if '%MQF_MQTEST%'=='Y' (
      call :LogEcho .  SKIP: Would have launched amqiregisterdotnet.cmd
    ) else (

      rem
      rem Set MQ_FILE_PATH otherwise causes problems applying maintenance to
      rem non-primary when a primary is also installed.
      rem

      rem -----------------------------------------------------------------------
      rem Note: bin, not bin64 at this level of the code.
      rem -----------------------------------------------------------------------
      set MQ_FILE_PATH=%MQF_MQROOTDIR%
      call %MQF_MQROOTDIR%\bin\amqiregisterdotnet.cmd

      rem Allow this to fail in case there is no .NET installed
      if errorlevel 1 (
        call :LogEcho .  WARNING - Potential failure to register the .net components.
        call :LogEcho .     Please rerun amqiregisterdotnet if applicable.
      )
      goto :EndReRegister
    )
  ) else (
    if '%MQF_PRIMARY%'=='Y' (
      call :LogEcho .Reregistering the .NET components via setmqinst
      if '%MQF_MQTEST%'=='Y' (
        call :LogEcho .  SKIP: Would have launched setmqinst -n %MQF_INSTALLATIONNAME% -r
      ) else (
        %MQF_MQBINDIR%\setmqinst.exe -n %MQF_INSTALLATIONNAME% -r
        if errorlevel 1 (
          call :LogEcho .  FAILURE - Failed to reapply the GAC changes via setmqinst
          set MQF_RC=32
          goto :EndReRegister
        )
      )
    ) else (
      call :LogEcho .  Not the primary install, so no GAC reregistering required.
    )
  )
:EndReRegister
  goto :EOF

:Return
  rem Special case -1 which enables a success return from anywhere in the
  rem code whilst dropping straight out of the processing.
  if '%MQF_RC%'=='-1' set MQF_RC=0

  rem Return the return code calculated by the program
  if not '%MQF_RC%'=='0' echo Failed. (%MQF_RC%)
  exit /b %MQF_RC%
